@@ -94,9 +94,17 @@
     Updated by      Richard Gooch   4-NOV-1999: Added compatibility entries for
   virtual console capture, serial and console devices.
 
-    Last updated by Richard Gooch   5-NOV-1999: Added compatibility entries for
+    Updated by      Richard Gooch   5-NOV-1999: Added compatibility entries for
   BSD pty devices.
 
+    Updated by      Richard Gooch   11-NOV-1999: Only show messages about
+  compatibility entries being created or destroyed when in trace mode.
+
+    Updated by      Richard Gooch   21-NOV-1999: Added compatibility entries
+  for joystick devices.
+
+    Last updated by Richard Gooch   6-DEC-1999: Change directory to devfs FS.
+
 
 */
 #include <unistd.h>
@@ -123,7 +131,7 @@
 #include <karma.h>
 
 
-#define VERSION "1.2.11"
+#define VERSION "1.2.12"
 #define CONFIG_FILE "/etc/devfsd.conf"
 #define MAX_ARGS (6 + 1)
 
@@ -199,7 +207,7 @@ EXTERN_FUNCTION (flag st_expr_expand,
 
 
 /*  Private functions  */
-static void setup_initial_entries (CONST char *mount_point);
+static void setup_initial_entries ();
 static void read_config (CONST char *filename, flag optional,
 			 unsigned long *event_mask);
 static void do_servicing (int fd, unsigned long event_mask);
@@ -219,7 +227,7 @@ static void sig_handler (int sig);
 static CONST char *get_variable (CONST char *variable, void *info);
 static void do_open_syslog ();
 static void do_scan_and_service (DIR *dp, CONST char *dirname);
-static flag make_symlink (const char *oldpath, char *newpath);
+static flag make_symlink (const char *oldpath, const char *newpath);
 
 
 /*  Private data  */
@@ -256,7 +264,6 @@ int main (int argc, char **argv)
     unsigned long event_mask = 0;
     struct sigaction new_action;
     struct stat statbuf;
-    char filename[STRING_LENGTH];
     static char usage[] = "devfsd mntpnt [-v] [-d] [-t num] [-D mask] [-fg]";
 
     if (argc < 2)
@@ -288,12 +295,15 @@ int main (int argc, char **argv)
     }
     if (trace_level > 0) no_syslog = TRUE;
     mount_point = argv[1];
-    strcpy (filename, argv[1]);
-    strcat (filename, "/.devfsd");
-    if ( ( fd = open (filename, O_RDONLY, 0) ) < 0 )
+    if (chdir (mount_point) != 0)
     {
-	fprintf (stderr, "Error opening file: \"%s\"\t%s\n",
-		 filename, ERRSTRING);
+	fprintf (stderr, "Error changing directory to: \"%s\"\t%s\n",
+		 mount_point, ERRSTRING);
+	exit (1);
+    }
+    if ( ( fd = open (".devfsd", O_RDONLY, 0) ) < 0 )
+    {
+	fprintf (stderr, "Error opening file: \".devfsd\"\t%s\n", ERRSTRING);
 	exit (1);
     }
     if (ioctl (fd, DEVFSDIOC_GET_PROTO_REV, &proto_rev) != 0)
@@ -301,7 +311,7 @@ int main (int argc, char **argv)
 	fprintf (stderr, "Error getting protocol revision\t%s\n", ERRSTRING);
 	exit (1);
     }
-    setup_initial_entries (mount_point);
+    setup_initial_entries ();
     if (!print_version && !debug_protocol && !debug_devfs)
     {
 	if (stat (CONFIG_FILE, &statbuf) != 0)
@@ -407,18 +417,12 @@ int main (int argc, char **argv)
 
 /*  Private functions follow  */
 
-static void setup_initial_entries (CONST char *mount_point)
+static void setup_initial_entries ()
 {
     struct initial_symlink_struct *curr;
-    char name[STRING_LENGTH];
 
     for (curr = initial_symlinks; curr->dest != NULL; ++curr)
-    {
-	strcpy (name, mount_point);
-	strcat (name, "/");
-	strcat (name, curr->name);
-	symlink (curr->dest, name);
-    }
+	symlink (curr->dest, curr->name);
 }   /*  End Function setup_initial_entries  */
 
 static void read_config (CONST char *filename, flag optional,
@@ -687,14 +691,11 @@ static void action_permissions (const struct devfsd_notify_struct *info,
 {
     mode_t new_mode;
     struct stat statbuf;
-    char path[STRING_LENGTH];
 
-    strcpy (path, mount_point);
-    strcat (path, "/");
-    strcat (path, info->devname);
-    if (lstat (path, &statbuf) != 0)
+    if (lstat (info->devname, &statbuf) != 0)
     {
-	SYSLOG (LOG_ERR, "error stat(2)ing: \"%s\"\t%s\n", path, ERRSTRING);
+	SYSLOG (LOG_ERR, "error stat(2)ing: \"%s\"\t%s\n",
+		info->devname, ERRSTRING);
 	SYSLOG (LOG_ERR, "exiting\n");
 	exit (1);
     }
@@ -703,34 +704,35 @@ static void action_permissions (const struct devfsd_notify_struct *info,
     if (trace_level > 2)
 	fprintf (stderr,
 		 "\tupdate permissions for \"%s\" from %05o to %05o, user.group from %d.%d to %d.%d\n",
-		 path, info->mode, new_mode, info->uid, info->gid,
+		 info->devname, info->mode, new_mode, info->uid, info->gid,
 		 entry->u.permissions.uid, entry->u.permissions.gid);
     if (new_mode != statbuf.st_mode)
     {
-	if (chmod (path, new_mode) != 0)
+	if (chmod (info->devname, new_mode) != 0)
 	{
 	    SYSLOG (LOG_ERR, "error changing mode for: \"%s\"\t%s\n",
-		    path, ERRSTRING);
+		    info->devname, ERRSTRING);
 	    SYSLOG (LOG_ERR, "exiting\n");
 	    exit (1);
 	}
 	if (trace_level > 2) fprintf (stderr,"Set permission %05o on \"%s\"\n",
-				      new_mode, path);
+				      new_mode, info->devname);
     }
     if ( (entry->u.permissions.uid != statbuf.st_uid) ||
 	 (entry->u.permissions.gid != statbuf.st_gid) )
     {
-	if (chown (path, entry->u.permissions.uid, entry->u.permissions.gid)
-	    != 0)
+	if (chown (info->devname, entry->u.permissions.uid,
+		   entry->u.permissions.gid) != 0)
 	{
 	    SYSLOG (LOG_ERR, "error changing mode for: \"%s\"\t%s\n",
-		    path, ERRSTRING);
+		    info->devname, ERRSTRING);
 	    SYSLOG (LOG_ERR, "exiting\n");
 	    exit (1);
 	}
 	if (trace_level > 2)
 	    fprintf (stderr, "Set user.group %d.%d on \"%s\"\n",
-		     entry->u.permissions.uid, entry->u.permissions.gid, path);
+		     entry->u.permissions.uid, entry->u.permissions.gid,
+		     info->devname);
     }
 }   /*  End Function action_permissions  */
 
@@ -797,7 +799,7 @@ static void action_compat (const struct devfsd_notify_struct *info,
     const char *compat_name = NULL;
     const char *dest_name = info->devname;
     char *ptr;
-    char compat_path[STRING_LENGTH], compat_buf[STRING_LENGTH];
+    char compat_buf[STRING_LENGTH];
     char dest_buf[STRING_LENGTH];
     static char function_name[] = "action_compat";
 
@@ -1063,6 +1065,20 @@ static void action_compat (const struct devfsd_notify_struct *info,
 		     pty1[index >> 4], pty2[index & 0x0f]);
 	    compat_name = compat_buf;
 	}
+	else if (strncmp (info->devname, "joysticks/", 10) == 0)
+	{
+	    int index;
+	    char *type = "";
+
+	    if (info->devname[10] == 'a') index = atoi (info->devname + 18);
+	    else
+	    {
+		index = atoi (info->devname + 17);
+		type = "d";
+	    }
+	    sprintf (compat_buf, "%sjs%d", type, index);
+	    compat_name = compat_buf;
+	}
 	break;
       case AC_MKNEWCOMPAT:
       case AC_RMNEWCOMPAT:
@@ -1204,27 +1220,27 @@ static void action_compat (const struct devfsd_notify_struct *info,
 	break;
     }
     if (compat_name == NULL) return;
-    sprintf (compat_path, "%s/%s", mount_point, compat_name);
     /*  Now decide what to do with it  */
     switch (action)
     {
       case AC_MKOLDCOMPAT:
       case AC_MKNEWCOMPAT:
-	if ( make_symlink (dest_name, compat_path) )
-	    SYSLOG (LOG_INFO, "made symlink: \"%s\" for dev: %u,%u\n",
-		    compat_path, info->major, info->minor);
+	if ( make_symlink (dest_name, compat_name) )
+	    if (trace_level > 1)
+		SYSLOG (LOG_INFO, "made symlink: \"%s\" for dev: %u,%u\n",
+			compat_name, info->major, info->minor);
 	break;
       case AC_RMOLDCOMPAT:
       case AC_RMNEWCOMPAT:
-	if (unlink (compat_path) != 0)
+	if (unlink (compat_name) != 0)
 	{
 	    SYSLOG (LOG_ERR, "%s: error unlinking: \"%s\"\t%s\n",
-		    function_name, compat_path, ERRSTRING);
+		    function_name, compat_name, ERRSTRING);
 	}
-	else SYSLOG (LOG_INFO, "unlinked: \"%s\"\n", compat_path);
+	else if (trace_level > 1)
+	    SYSLOG (LOG_INFO, "unlinked: \"%s\"\n", compat_name);
 	break;
     }
-    if (!syslog_is_open) usleep (10000);
 }   /*  End Function action_compat  */
 
 static void free_config ()
@@ -1482,7 +1498,7 @@ static void do_scan_and_service (DIR *dp, CONST char *dirname)
     }
 }   /*  End Function do_scan_and_service  */
 
-static flag make_symlink (const char *oldpath, char *newpath)
+static flag make_symlink (const char *oldpath, const char *newpath)
 /*  [SUMMARY] Create a symlink, creating intervening directories as required.
     <oldpath> The string contained in the symlink.
     <newpath> The name of the new symlink.
@@ -1491,25 +1507,27 @@ static flag make_symlink (const char *oldpath, char *newpath)
 {
     char ch;
     char *ptr1, *ptr2;
+    char newpath_copy[STRING_LENGTH];
     static char function_name[] = "make_symlink";
 
-    ptr1 = newpath;
+    strcpy (newpath_copy, newpath);
+    ptr1 = newpath_copy;
     while (TRUE)
     {
 	if ( ( ptr2 = strchr (ptr1 + 1, '/') ) == NULL ) break;
 	ch = *ptr2;
 	*ptr2 = '\0';
-	if (access (newpath, F_OK) == 0)
+	if (access (newpath_copy, F_OK) == 0)
 	{
 	    *ptr2 = ch;
 	    ptr1 = ptr2;
 	    continue;
 	}
-	if (mkdir (newpath,
+	if (mkdir (newpath_copy,
 		   S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0)
 	{
 	    SYSLOG (LOG_ERR, "Error making directory \"%s\"\t%s\n",
-		    newpath, ERRSTRING);
+		    newpath_copy, ERRSTRING);
 	    *ptr2 = ch;
 	    return (FALSE);
 	}
@@ -1519,7 +1537,10 @@ static flag make_symlink (const char *oldpath, char *newpath)
     if (symlink (oldpath, newpath) != 0)
     {
 	if (errno == EEXIST)
-	    SYSLOG (LOG_INFO, "symlink: \"%s\" already exists\n", newpath);
+	{
+	    if (trace_level > 1)
+		SYSLOG (LOG_INFO, "symlink: \"%s\" already exists\n", newpath);
+	}
 	else
 	    SYSLOG (LOG_ERR, "%s: error creating symlink: \"%s\"\t%s\n",
 		    function_name, newpath, ERRSTRING);
